/* Date: 03/02/2013
 * Copyright 2013 Pablo Cremades
 * Author: Pablo Cremades
 *  e-mail: pablocremades@gmail.com
 * Description:
 */
/* *************************************************************************
*	This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.*
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY* without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* *************************************************************************/

#include "include/Oz47.h"
#include <Arduino.h>

Oz47::Oz47()
{
    //ctor
}

Oz47::~Oz47()
{
    //dtor
}

//Put sensor in automatic mode.
void Oz47::Mauto()
{
    Serial.print("{S}");
}

void Oz47::Mmanual()
{
    Serial.print("{K}");
}

/* This function performs a measurement and process the answer from the sensor.
* Note that this functions takes a lot of CPU time while communicating with the sensor.*/
void Oz47::Sense()
{
    char input_buff[CMD_LENGTH];
    int j;

    Serial.print("{M}");    //Send measurement command.
    delay(100);     //Wait for answer.

    j = 0;
    //Start retrieving answer.
    while( Serial.available() )    //Checks if there is incoming data from sensor.
    {
        input_buff[j] = Serial.read();
        j++;
    }
    input_buff[j] = '\0';
    while( (input_buff[j] != 'M') & (j > 0) )   //Rewind to answer begining.
    {
        j--;
    }

    //Process answer.
    if( (j > 0) & (input_buff[17] == '}') )
    {
        //Serial.print("Joya");
        j++;
        ozone1 = input_buff[j] - 0x30; j++; ozone1 = (ozone1 << 4) + input_buff[j] - 0x30;
        j++;
        ozone2 = input_buff[j] - 0x30; j++; ozone2 = (ozone2 << 4) + input_buff[j] - 0x30;
        j++;
        Res1 = input_buff[j] - 0x30; j++; Res1 = (Res1 << 4) + input_buff[j] - 0x30; j++; Res1 = (Res1 << 4) + input_buff[j] - 0x30;
        j++;
        Res2 = input_buff[j] - 0x30; j++; Res2 = (Res2 << 4) + input_buff[j] - 0x30; j++; Res2 = (Res2 << 4) + input_buff[j] - 0x30;
        j++;
        temp = input_buff[j] - 0x30; j++; temp = (temp << 4) + input_buff[j] - 0x30; j++; temp = (temp << 4) + input_buff[j] - 0x30;
        tempf = (float)temp / 10 - 40;
        j++;
        humd = input_buff[j] - 0x30; j++; humd = (humd << 4) + input_buff[j] - 0x30;
    }
    else
    {
        //Return some error codes.
        //CHECK the types of the variables to optimize their size and adjust error codes accordingly!!!!!!!!!----------------------------------------------->
        //Serial.println("Algo mal.");
        ozone1 = -999;
        ozone2 = -999;
        Res1 = -999;
        Res2 = -999;
        tempf = -99.9;
        humd = -999;
    }
}

int Oz47::getRes()
{
    return Res1;
}

int Oz47::getOz()
{
    return ozone1;
}

float Oz47::getTemp()
{
    return tempf;
}

int Oz47::getRH()
{
    return humd;
}
